package hu.bme.mit.serializer;

import com.google.inject.Inject;
import com.google.inject.Provider;
import hu.bme.mit.services.SocialNetworkLanguageGrammarAccess;
import hu.bme.mit.socialNetworkLanguage.Acquaintance;
import hu.bme.mit.socialNetworkLanguage.Community;
import hu.bme.mit.socialNetworkLanguage.Person;
import hu.bme.mit.socialNetworkLanguage.SocialNetwork;
import hu.bme.mit.socialNetworkLanguage.SocialNetworkLanguagePackage;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtext.serializer.acceptor.ISemanticSequenceAcceptor;
import org.eclipse.xtext.serializer.acceptor.SequenceFeeder;
import org.eclipse.xtext.serializer.diagnostic.ISemanticSequencerDiagnosticProvider;
import org.eclipse.xtext.serializer.diagnostic.ISerializationDiagnostic.Acceptor;
import org.eclipse.xtext.serializer.sequencer.AbstractDelegatingSemanticSequencer;
import org.eclipse.xtext.serializer.sequencer.GenericSequencer;
import org.eclipse.xtext.serializer.sequencer.ISemanticNodeProvider.INodesForEObjectProvider;
import org.eclipse.xtext.serializer.sequencer.ISemanticSequencer;
import org.eclipse.xtext.serializer.sequencer.ITransientValueService;
import org.eclipse.xtext.serializer.sequencer.ITransientValueService.ValueTransient;

@SuppressWarnings("all")
public class SocialNetworkLanguageSemanticSequencer extends AbstractDelegatingSemanticSequencer {

	@Inject
	private SocialNetworkLanguageGrammarAccess grammarAccess;
	
	public void createSequence(EObject context, EObject semanticObject) {
		if(semanticObject.eClass().getEPackage() == SocialNetworkLanguagePackage.eINSTANCE) switch(semanticObject.eClass().getClassifierID()) {
			case SocialNetworkLanguagePackage.ACQUAINTANCE:
				if(context == grammarAccess.getAcquaintanceRule()) {
					sequence_Acquaintance(context, (Acquaintance) semanticObject); 
					return; 
				}
				else break;
			case SocialNetworkLanguagePackage.COMMUNITY:
				if(context == grammarAccess.getCommunityRule() ||
				   context == grammarAccess.getEntityRule()) {
					sequence_Community(context, (Community) semanticObject); 
					return; 
				}
				else break;
			case SocialNetworkLanguagePackage.PERSON:
				if(context == grammarAccess.getEntityRule() ||
				   context == grammarAccess.getPersonRule()) {
					sequence_Person(context, (Person) semanticObject); 
					return; 
				}
				else break;
			case SocialNetworkLanguagePackage.SOCIAL_NETWORK:
				if(context == grammarAccess.getSocialNetworkRule()) {
					sequence_SocialNetwork(context, (SocialNetwork) semanticObject); 
					return; 
				}
				else break;
			}
		if (errorAcceptor != null) errorAcceptor.accept(diagnosticProvider.createInvalidContextOrTypeDiagnostic(semanticObject, context));
	}
	
	/**
	 * Constraint:
	 *     (source=[Person|ID] type=AcquaintanceType target=[Person|ID])
	 */
	protected void sequence_Acquaintance(EObject context, Acquaintance semanticObject) {
		if(errorAcceptor != null) {
			if(transientValues.isValueTransient(semanticObject, SocialNetworkLanguagePackage.Literals.ACQUAINTANCE__SOURCE) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, SocialNetworkLanguagePackage.Literals.ACQUAINTANCE__SOURCE));
			if(transientValues.isValueTransient(semanticObject, SocialNetworkLanguagePackage.Literals.ACQUAINTANCE__TYPE) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, SocialNetworkLanguagePackage.Literals.ACQUAINTANCE__TYPE));
			if(transientValues.isValueTransient(semanticObject, SocialNetworkLanguagePackage.Literals.ACQUAINTANCE__TARGET) == ValueTransient.YES)
				errorAcceptor.accept(diagnosticProvider.createFeatureValueMissing(semanticObject, SocialNetworkLanguagePackage.Literals.ACQUAINTANCE__TARGET));
		}
		INodesForEObjectProvider nodes = createNodeProvider(semanticObject);
		SequenceFeeder feeder = createSequencerFeeder(semanticObject, nodes);
		feeder.accept(grammarAccess.getAcquaintanceAccess().getSourcePersonIDTerminalRuleCall_1_0_1(), semanticObject.getSource());
		feeder.accept(grammarAccess.getAcquaintanceAccess().getTypeAcquaintanceTypeEnumRuleCall_3_0(), semanticObject.getType());
		feeder.accept(grammarAccess.getAcquaintanceAccess().getTargetPersonIDTerminalRuleCall_5_0_1(), semanticObject.getTarget());
		feeder.finish();
	}
	
	
	/**
	 * Constraint:
	 *     (name=ID subcommunities+=Community*)
	 */
	protected void sequence_Community(EObject context, Community semanticObject) {
		genericSequencer.createSequence(context, semanticObject);
	}
	
	
	/**
	 * Constraint:
	 *     (name=ID sex=Sex (memberships+=[Community|ID] memberships+=[Community|ID]*)?)
	 */
	protected void sequence_Person(EObject context, Person semanticObject) {
		genericSequencer.createSequence(context, semanticObject);
	}
	
	
	/**
	 * Constraint:
	 *     (entities+=Entity | acquaintances+=Acquaintance)+
	 */
	protected void sequence_SocialNetwork(EObject context, SocialNetwork semanticObject) {
		genericSequencer.createSequence(context, semanticObject);
	}
}
